/**********************************************\
*                                              *
*  Linux Kernel Driver for PCL PCI(e) Devices  *
*                                              *
\**********************************************/

// $Id: avpcldrv.h 2123 2023-10-05 14:32:37Z J.Loh $
// Version: 5.45 (c) 1995-2023 PLURA Europe GmbH

#ifndef _AVPCL_H
#define _AVPCL_H
#include <linux/ioctl.h>
#include "driver.h"

#define AVPCL_IOC_MAGIC	0x17

/*
	define ioctl-commands
*/
#define AVPCL_IOC_READDATA0  _IOR(AVPCL_IOC_MAGIC, 1,  int)
#define AVPCL_IOC_READDATA1  _IOR(AVPCL_IOC_MAGIC, 2,  int)
#define AVPCL_IOC_READDATA2  _IOR(AVPCL_IOC_MAGIC, 3,  int)
#define AVPCL_IOC_READDATA3  _IOR(AVPCL_IOC_MAGIC, 4,  int)
#define AVPCL_IOC_READDATA4  _IOR(AVPCL_IOC_MAGIC, 5,  int)
#define AVPCL_IOC_READDATA5  _IOR(AVPCL_IOC_MAGIC, 6,  int)
#define AVPCL_IOC_READDATA6  _IOR(AVPCL_IOC_MAGIC, 7,  int)
#define AVPCL_IOC_READDATA7  _IOR(AVPCL_IOC_MAGIC, 8,  int)
#define AVPCL_IOC_READDATA8  _IOR(AVPCL_IOC_MAGIC, 9,  int)
#define AVPCL_IOC_READDATA9  _IOR(AVPCL_IOC_MAGIC, 10, int)
#define AVPCL_IOC_READDATAA  _IOR(AVPCL_IOC_MAGIC, 11, int)

#define AVPCL_IOC_READCMDR   _IOR(AVPCL_IOC_MAGIC, 12, int)
#define AVPCL_IOC_READACK    _IOR(AVPCL_IOC_MAGIC, 13, int)

#define AVPCL_IOC_WRITECMD    _IOR(AVPCL_IOC_MAGIC, 14, int)
#define AVPCL_IOC_WRITECMDD0  _IOR(AVPCL_IOC_MAGIC, 15, int)
#define AVPCL_IOC_WRITECMDD1  _IOR(AVPCL_IOC_MAGIC, 16, int)
#define AVPCL_IOC_WRITECFG    _IOR(AVPCL_IOC_MAGIC, 17, int)
#define AVPCL_IOC_WRITEINTACK _IOR(AVPCL_IOC_MAGIC, 18, int)

#define AVPCL_IOC_READVERSION _IOR(AVPCL_IOC_MAGIC, 19, int)
#define AVPCL_IOC_READINTFLGL _IOR(AVPCL_IOC_MAGIC, 20, int)
#define AVPCL_IOC_READINTFLGH _IOR(AVPCL_IOC_MAGIC, 21, int)
#define AVPCL_IOC_WRITEINTCSR _IOR(AVPCL_IOC_MAGIC, 22, int)

#define AVPCL_IOC_REGISTER_INT		_IOR(AVPCL_IOC_MAGIC, 23, int)
#define AVPCL_IOC_UNREGISTER_INT	_IOR(AVPCL_IOC_MAGIC, 24, int)

#define AVPCL_RETR_DEVID 			_IOR(AVPCL_IOC_MAGIC, 25, int)
#define AVPCL_RETR_SUBSYSDEVID 		_IOR(AVPCL_IOC_MAGIC, 26, int)
#define AVPCL_RETR_BUS				_IOR(AVPCL_IOC_MAGIC, 27, int)

#define AVPCL_READ_BAR	 			_IOR(AVPCL_IOC_MAGIC, 28, int)
#define AVPCL_WRITE_BAR 			_IOR(AVPCL_IOC_MAGIC, 29, int)
#define AVPCL_READ_CONFIG 			_IOR(AVPCL_IOC_MAGIC, 30, int)

#define AVPCL_RETR_DEVINFO 			_IOR(AVPCL_IOC_MAGIC, 31, int)
#define AVPCL_BRIDGE_RETR_DEVINFO	_IOR(AVPCL_IOC_MAGIC, 32, int)

#define AVPCL_BRIDGE_READ_CONFIG 	_IOR(AVPCL_IOC_MAGIC, 33, int)
#define AVPCL_BRIDGE_WRITE_CONFIG	_IOR(AVPCL_IOC_MAGIC, 34, int)
#define AVPCL_TEST_BRIDGE 			_IOR(AVPCL_IOC_MAGIC, 35, int) // Check if a handle is a bridge
#define AVPCL_BRIDGE_RETR_DEVID		_IOR(AVPCL_IOC_MAGIC, 36, int)
#define AVPCL_BRIDGE_READ_BAR	 	_IOR(AVPCL_IOC_MAGIC, 37, int)
#define AVPCL_BRIDGE_WRITE_BAR	 	_IOR(AVPCL_IOC_MAGIC, 38, int)

#define IS_BRIDGE_COMMAND(_cmd) (cmd == AVPCL_BRIDGE_READ_CONFIG ||\
								 cmd == AVPCL_BRIDGE_WRITE_CONFIG ||\
								 cmd == AVPCL_TEST_BRIDGE ||\
								 cmd == AVPCL_BRIDGE_RETR_DEVID ||\
								 cmd == AVPCL_BRIDGE_READ_BAR || \
								 cmd == AVPCL_BRIDGE_WRITE_BAR)

// Data exchange structs
typedef struct {
	DWORD offset;
	DWORD size;
	void *result;
} avpcl_read_config_t;	// Read config space

typedef struct {
	bar_t bar;
	DWORD offset;
	DWORD size;
	void *result;
} avpcl_read_bar_t;			// Read bar

typedef struct {
	bar_t bar;
	DWORD offset;
	DWORD size;
	void *data;
} avpcl_write_bar_t;			// Write bar

#endif // _AVPCL_H
