/**********************************************\
*                                              *
*  Linux Kernel Driver for PCL PCI(e) Devices  *
*                                              *
*        Part of the shared library            *
*                                              *
\**********************************************/

// $Id: driver.h 2123 2023-10-05 14:32:37Z J.Loh $
// Version: 5.45 (c) 1995-2023 PLURA Europe GmbH

#ifndef AVPCL_DRIVER_H
#define AVPCL_DRIVER_H

//---------------------------------------------------------------------------

#ifdef LINUX
#define DEBUG_PRINTF(...) printf(__VA_ARGS__)
#else
#define DEBUG_PRINTF(...)
#endif
//---------------------------------------------------------------------------

#ifdef LINUX
#define AVPCL6_DEFAULT_VENDOR_ID 0x10B5 /* Vendor ID */
#define AV_SIGNAL 34 // Hard coded signal due to different mapping in kernel and user space
#endif

//---------------------------------------------------------------------------

#define PROC_FILENAME_INC "avpcl32CounterInc"
#define PROC_FILENAME_DEC "avpcl32CounterDec"

//---------------------------------------------------------------------------

enum {
	// CFG write
	PCI_CFG_RESET_M		= 0x01,
	PCI_CFG_ACK_M		= 0x02,
	PCI_CFG_PSEN_M		= 0x04,
	PCI_CFG_PLINTFF_M	= 0x08,

	// CFG read
	PCI_CFG_VBI_16K_M	= (1 << 2),	// 16K VBI memory

	// INTENA
	PCI_INTENA_PLINT1_M	= 0x01,	// bit 0	1=enable plint1, 0=disable plint1
};

//---------------------------------------------------------------------------

enum {
	PCI_DEFAULT_DEVICE_ID_5V   			= AVPCL_DEVICE_ID_5V,	// 0x9050
	PCI_DEFAULT_DEVICE_ID_3V   			= AVPCL_DEVICE_ID_3V,	// 0x9030
	PCI_DEFAULT_DEVICE_ID_PCIE 			= AVPCL_DEVICE_ID_E,	// 0x9056
#ifdef PCL_OPT_FLASH
	PCI_DEFAULT_DEVICE_ID_PCIE_BRIDGE 	= 0x8111,				// 0x8111
#endif
};

//---------------------------------------------------------------------------

enum { PCI_DEFAULT_VENDOR_ID = AVPCL6_DEFAULT_VENDOR_ID };

//---------------------------------------------------------------------------

#ifdef LINUX
void 	pclDestroyAllSharedMemory(void);
int 	pclGetActiveProcessCount(void); // returns 0 on error.
BOOL 	pclIsFlashActive(void);
// void pclSetFlashActive(BOOL bActive); siehe unten
#endif

//---------------------------------------------------------------------------

#ifdef PCL_OPT_FLASH
#define PCI_BRIDGES PCL_CARDS
typedef struct {
	WORD deviceId;
	BYTE bus;
	unsigned long 		bar0Start;
	unsigned long 		bar0Length;
	unsigned long 		bar2Start;
	unsigned long 		bar2Length;
} TDeviceInfo;
int pclGetDeviceInfo(int hdl, TDeviceInfo _PCL_OUT *deviceInfo); 	// Bridge enabled
int pclGetDeviceId(int hdl, WORD _PCL_OUT *deviceId); 				// Bridge enabled
int pclGetBus(int hdl, BYTE _PCL_OUT *bus);
int pclGetSubsystemDeviceId(int hdl, WORD _PCL_OUT *subsystemDeviceId);
typedef enum { 
	AVPCL_BAR_0 = 0,
	AVPCL_BAR_2 = 2
} bar_t;
int pclReadOffset8  (int hdl, BYTE bar, UINT offset, BYTE _PCL_OUT *result); 	// Bridge enabled
int pclReadOffset32 (int hdl, BYTE bar, UINT offset, DWORD _PCL_OUT *result);	// Bridge enabled
int pclWriteOffset8 (int hdl, BYTE bar, UINT offset, BYTE  data); 			// Bridge enabled
int pclWriteOffset32(int hdl, BYTE bar, UINT offset, DWORD data); 			// Bridge enabled
int pclConfigDump(int hdl, UINT offset, UINT size, void _PCL_OUT *result); 	// Bridge enabled
//int pclConfigWriteOffset32(int hdl, UINT offset, DWORD data); // Bridge only
BYTE pclReadVersion(int hdl);
int pclOpenBridge(int *hdl, UINT base);
BOOL pclSetFlashActive(BOOL bActive); // return FALSE on error.
#endif

#endif // AVPCL_DRIVER_H
