/**********************************************\
*                                              *
*  Linux Kernel Driver for PCL PCI(e) Devices  *
*                                              *
*        Part of the shared library            *
*                                              *
\**********************************************/

// $Id: funcs.h 2123 2023-10-05 14:32:37Z J.Loh $
// Version: 5.45 (c) 1995-2023 PLURA Europe GmbH

#ifndef _FUNCS_H
#define _FUNCS_H

#ifdef PCL_DEBUG_LOG
#	include <stdio.h>
#endif

#ifdef _WIN32
#	include "pci_lib.h"
#endif
#ifdef LINUX
#	include "pci_lib_linux.h"
#	include <pthread.h>
#endif

#ifdef PCL_METERED_SECTION
#	include "MeteredSection.h"
#endif // PCL_METERED_SECTION

#ifdef PCL_FAST_MUTEX
#	include "FastMutex.h"
#endif // PCL_FAST_MUTEX

#include "driver.h"

//---------------------------------------------------------------------------

#define NELT(v)				(sizeof(v) / sizeof(v[0]))

#define SIZEOF_MEMBER(s, m)	sizeof(((s *)0)->m)

#define ASSERT_STATIC(b)	switch(0){case 0:case b:;}
						//	typedef char static_assertion[(b) ? 1 : -1]
//---------------------------------------------------------------------------

#ifdef LINUX
#define MAX_PATH (256)
#define _countof(v) NELT(v)

#ifdef PCL_USE_SHM_MUTEX
typedef struct {
	pthread_mutex_t mutex;
	BOOL bMutexValid;
	struct {
		int flash		: 1;
		int _reserved	: 7;
	} flags;
} TMutex;
#endif // PCL_USE_SHM_MUTEX

#endif

//---------------------------------------------------------------------------

typedef enum {
	TYPE_UNUSED = 0,	// handle frei
	obsolete_TYPE_ISA,	// pcl-isa
	TYPE_PCI,			// pcl-pci
	TYPE_RESERVED,		// reserviert
#ifdef PCL_MULTI_MODE
	TYPE_MULTI,			// multiple access
	TYPE_MULTI_REOPEN,	// multiple access, re-open handle
	TYPE_MULTI_WAIT,	// multiple access, wait for re-open
#else
	_reserved_for_TYPE_MULTI,
	_reserved_for_TYPE_MULTI_REOPEN,
	_reserved_for_TYPE_MULTI_WAIT,
#endif
} THdlType;

//---------------------------------------------------------------------------

#ifdef PCL_MULTI_MODE

typedef struct {
	DWORD counter;	/* number of interrupts received */
} TPciMultiHdlTc;

#endif // PCL_MULTI_MODE

//---------------------------------------------------------------------------

typedef union {
	void *p;
#if defined(_WIN32) || defined(LINUX)
	PCL_INT_HANDLER f;
#endif
} TPciIntHandler;

//---------------------------------------------------------------------------

typedef struct {
	int _dummy;
	struct {
		THdlType type;
		//--------------------------
		int card;
		int imask;
#ifdef PCL_OPT_ANC				/* read user defined ancillary data */
		struct {
			int imask;
			int index;
		} anc;
#endif
#ifdef PCL_DEBUG_LOG
		FILE *file;
#endif // PCL_DEBUG_LOG
//		BOOL porSupported;
#ifdef PCL_METERED_SECTION
		LPMETERED_SECTION section;
#endif // PCL_METERED_SECTION

		TPciIntHandler intHandler;

#ifdef PCL_MULTI_MODE
		struct {
//			bool enable;
			int hdl;
			bool main;
			struct {
				TPciMultiHdlTc mixed, ltc, vitc, atcLtc, atcVitc, hancLtc, hancVitc;
			} tc;
		} multi;
#endif // _WIN32
	} v[PCL_HANDLES];
} TPciHdl;

extern TPciHdl *pgPciHdl;

//---------------------------------------------------------------------------

// #define PCL_CARDS (PCL_BASE_PCI_LAST - PCL_BASE_PCI_OFS) moved to driver.h

typedef struct {
	int _dummy;
	struct {
		int open;	// open count

		WORD wVendorID;
		WORD wDeviceID;
		WORD wSubsystemVendorID;
		WORD wSubsystemDeviceID;

		int imask;

		PCI_HANDLE hPCI;
#ifdef LINUX
		PCI_INT_HANDLER 	hIntThread;
		pthread_t 			intThreadId;
#endif

#ifdef _WIN32
		WDC_DEVICE_HANDLE hDev;
#endif
		BOOL irq16;
		struct {
			PCL_TIME_STAMP abs;
			PCL_TIME_STAMP freq;
		} perf;
		BOOL bHasIntffRead;
		int logAddr;
#ifdef PCL_OPT_ANC				/* read user defined ancillary data */
//		BOOL bHasMultiAncRead;
#endif

#ifdef PCL_WAKEUP_PERSISTENT
		struct {
			struct {
				PCL_BOOL use;
				int rmask;
			} readerEnable;
			struct {
				PCL_BOOL use;
				int line1;
				int line2;
				int vline;
			} vitcSetup;
			struct {
				PCL_BOOL use;
				int rmask;
			} mixedEnable;
			struct {
				PCL_BOOL use;
				int prior;
			} priority;
			struct {
				PCL_BOOL use;
				int tcfr;
			} tcFrames;
			struct {
				PCL_BOOL use;
				int ltcfr;
			} ltcFrames;
			struct {
				PCL_BOOL use;
				int vitcfr;
			} vitcFrames;
			struct {
				PCL_BOOL use;
				int vlmode;
				int vlev;
			} vitcLevelControl;
			struct {
				PCL_BOOL use;
				int ecmode;
			} ltcError;
			struct {
				PCL_BOOL use;
				int ecmode;
			} vitcError;
			struct {
				PCL_BOOL use;
				int fwmode;
			} flyWheel;
			struct {
				PCL_BOOL use;
				int gpi;
				int reg;
				int mode;
				PCL_TC time;
			} setGpiEventTime;
			struct {
				PCL_BOOL use;
				int mode;
			} setRealtimeMode;
			struct {
				PCL_BOOL use;
				int ecmode;
			} atcLtcError;
			struct {
				PCL_BOOL use;
				int ecmode;
			} atcVitc1Error;
			struct {
				PCL_BOOL use;
				int ecmode;
			} atcVitc2Error;
			struct {
				PCL_BOOL use;
				int ecmode;
			} hancLtcError;
			struct {
				PCL_BOOL use;
				int ecmode;
			} hancVitcError;
		} persist;
#endif
	} v[PCL_CARDS];
} TPciCard;

extern TPciCard gPciCard;

//---------------------------------------------------------------------------

typedef struct {
	int _dummy;
	struct {

		BOOL locked;
#ifdef PCL_FAST_MUTEX
		CFastMutex mutex;
#endif // PCL_FAST_MUTEX

#if defined(PCL_USE_SHM_MUTEX) && defined(LINUX)
		TMutex *pMutex;
		int shmid; // shared memory id
#endif

	} v[PCL_CARDS];
} TPciCardLocal;

extern TPciCardLocal gPciCardLocal;

//---------------------------------------------------------------------------

#ifdef PCL_MULTI_MODE

typedef struct {
	PCL_TC time;
	PCL_USER user;
	int	bits, frame, newd, stat;
	int result;
	DWORD counter;	/* number of interrupts received */
//	PCL_INT_RESULT intResult;
} TPciMultiCardTc;

#endif // PCL_MULTI_MODE

typedef struct {
	int _dummy;
	struct {
		DWORD processId;
#if 0
		struct {
			PCI_STRUCT sPCI;
			AVPCL6_DEV_CTX sDevCtx;
			WDC_DEVICE sDev;
			WD_TRANSFER sTrans;
			INT_THREAD_DATA sThread;
			V_WDC_ADDR_DESC vAddrDesc;
			struct {
				thread_handle_t h;
				thread_struct_t t;
			} thread;
		} malloc;
#endif
#ifdef PCL_MULTI_MODE
		struct {
//			bool enable;
/*
			int hdl;
			bool main;
*/
			struct {
				TPciMultiCardTc mixed, ltc, vitc, atcLtc, atcVitc, hancLtc, hancVitc;
			} tc;
			struct {
				int ident;		// software identification
				int firmware;	// firmware revision number
				int dll;		// dll revision number
			} pclGetVersion;
			struct {
				int ident1;	// identifier from byte $A
				int ident2;	// identifier from byte $B
				int ident3;	// identifier from byte $C
			} pclGetVerSpecial;
		} multi;
#endif // PCL_MULTI_MODE
	} v[PCL_CARDS];
} TPciCardShared;

extern TPciCardShared *pgPciCardShared;

//---------------------------------------------------------------------------

typedef struct {
	TPciCardShared PciCardShared;
	TPciHdl PciHdl;
} TPciShared;

//---------------------------------------------------------------------------

#ifdef _WIN32
#ifdef PCL_NAMED_SHARED
#	define PCI_SHARED_MEMORY_NAME "USR_AVPCLPCI_SHARED_MEMORY"
#endif // PCL_NAMED_SHARED
#endif // _WIN32

//---------------------------------------------------------------------------

#ifdef PCL_FAST_MUTEX

typedef struct {
	int _dummy;
	CFastMutex mutex;
} TPciHdlMutex;

extern TPciHdlMutex gPciHdlMutex;

#endif

#if defined(LINUX) && defined(PCL_OPT_FLASH)
typedef struct {
	struct {
		PCI_HANDLE hPCI;
		int card;
	} v[PCI_BRIDGES];
} TPclBridges;
extern TPclBridges gPclBridges;
#endif

//---------------------------------------------------------------------------

#ifdef _WIN32

typedef struct {
	int _dummy;
	LONG value;
} TDllUseCount;

extern TDllUseCount gDllUseCount;

#endif // _WIN32

//---------------------------------------------------------------------------

#define PCI_METERED_SECTION_NAME "USR_AVPCLPCI_METERED_SECTION_%d"

#define PCI_METERED_SECTION_TIMEOUT 1000	// milliseconds

//---------------------------------------------------------------------------

#define CHECK_PCI_HANDLE(_hPCI) \
	if (_hPCI == NULL) { \
		pclErrorString = "Handle is NULL\n"; \
		return(PCL_ERR_INV_HDL); \
	} // check if handle '_hPCI' is plausible

//---------------------------------------------------------------------------

#define CHECK_CARD(_card) do { \
	assert(pgPciCardShared); \
	if ((_card) < 0 || (_card) >= PCL_CARDS) { \
		/*pclErrorString = "Parameter '" #_card "' is out of range\n";*/ \
		assert(FALSE); \
		/*return(PCL_ERR_INV_HDL);*/ \
	} \
} while (0)

//---------------------------------------------------------------------------

#ifdef _WIN32

#define CHECK_HANDLE(_hdl) do { \
	assert(pgPciHdl); \
	if ((_hdl) < 0 || (_hdl) >= PCL_HANDLES) { \
		pclErrorString = "Parameter '" #_hdl "' is out of range\n"; \
		assert(FALSE); \
		return(PCL_ERR_INV_HDL); \
	} \
} while (0)

#ifdef PCL_MULTI_MODE

#define PCI_GET_HANDLE(_hdl, _func, _param, _hPCI, _card) \
	CHECK_HANDLE(_hdl); \
	int _card; \
	if (pgPciHdl->v[_hdl].type == TYPE_MULTI_REOPEN) { \
		int result; \
		if ((result = pclPciReopenMulti(hdl)) != PCL_OK) { \
			return(result); \
		} \
	} \
	if (pgPciHdl->v[_hdl].type == TYPE_MULTI) { \
		/*APPEND_DEBUG_LOG(hdl, "< " #_func "Multi" #_param);*/ \
		return(_func##Multi _param); \
	} \
	if (pgPciHdl->v[_hdl].type == TYPE_MULTI_WAIT) { \
		pclErrorString = "Handle is currently re-opened\n"; \
		return(PCL_NO_NEWD); \
	} else { \
		assert(pgPciHdl->v[_hdl].type == TYPE_PCI); \
		_card = pgPciHdl->v[_hdl].card; \
		CHECK_CARD(_card); \
	} \
	PCI_HANDLE _hPCI = gPciCard.v[_card].hPCI; \
	CHECK_PCI_HANDLE(_hPCI); \
	pclErrorString = pclPciGetErrorString();

#else // PCL_MULTI_MODE

#define PCI_GET_HANDLE(_hdl, _func, _param, _hPCI, _card) \
	CHECK_HANDLE(_hdl); \
	int _card; \
	{ \
		assert(pgPciHdl->v[_hdl].type == TYPE_PCI); \
		_card = pgPciHdl->v[_hdl].card; \
		CHECK_CARD(_card); \
	} \
	PCI_HANDLE _hPCI = gPciCard.v[_card].hPCI; \
	CHECK_PCI_HANDLE(_hPCI); \
	pclErrorString = pclPciGetErrorString();

#endif // PCL_MULTI_MODE

#define PCI_GET_HANDLE_BRIDGE(_hdl, _func, _param, _hPCI, _card, _useBridge) \
	assert(!_useBridge); \
	PCI_GET_HANDLE(_hdl, _func, _param, _hPCI, _card)

#define PCI_GET_HANDLE_MULTI(_hdl, _func, _param, _hPCI, _card) \
	CHECK_HANDLE(_hdl); \
	assert(pgPciHdl->v[_hdl].type == TYPE_MULTI);
//	assert(pgPciHdl->v[pgPciHdl->v[_hdl].multi.hdl].type == TYPE_PCI);

#endif // _WIN32

//---------------------------------------------------------------------------

#ifdef LINUX

#ifdef PCL_OPT_FLASH
	BOOL IsBridge(int hdl);
	int ConvertHandle(int hdl);
#else
	inline BOOL IsBridge(int hdl)	{	return FALSE;	}
	inline int ConvertHandle(int hdl) { return hdl; };
#endif

#define CHECK_HANDLE_BRIDGE(_hdl, _useBridge) \
	if ((_useBridge) ?  ((_hdl) < PCL_HANDLES || (_hdl) >= (NELT(gPclBridges.v) + PCL_HANDLES)) : \
						((_hdl) < 0 || (_hdl) >= PCL_HANDLES)) { \
		pclErrorString = "Parameter '" #_hdl "' is out of range\n"; \
		assert(FALSE); \
		return(PCL_ERR_INV_HDL); \
	}
#define CHECK_HANDLE(_hdl) \
		CHECK_HANDLE_BRIDGE(_hdl, FALSE)

#define PCI_GET_HANDLE_BRIDGE(_hdl, _func, _param, _hPCI, _card, _useBridge) \
	CHECK_HANDLE_BRIDGE(_hdl, _useBridge); \
	int _card = (_useBridge) ? gPclBridges.v[ConvertHandle(_hdl)].card : pgPciHdl->v[_hdl].card; \
	CHECK_CARD(_card); \
	PCI_HANDLE _hPCI = (_useBridge) ? gPclBridges.v[ConvertHandle(_hdl)].hPCI : gPciCard.v[_card].hPCI; \
	(void) (_card); \
	if (!(_useBridge) && pgPciHdl->v[_hdl].type != TYPE_PCI) { \
		_hPCI = -1; \
		pclErrorString = "Handle '" #_hdl "' is invalid\n"; \
	} else {\
		pclErrorString = pclPciGetErrorString();\
	}

#define PCI_GET_HANDLE(_hdl, _func, _param, _hPCI, _card) \
		PCI_GET_HANDLE_BRIDGE(_hdl, _func, _param, _hPCI, _card, FALSE)

#endif // LINUX

//---------------------------------------------------------------------------

BOOL pclPciLock(int hdl);

void pclPciUnlock(int hdl);

//---------------------------------------------------------------------------

#define PCI_GET_HANDLE_LOCK_BRIDGE(_hdl, _func, _param, _hPCI, _card, _useBridge) \
	PCI_GET_HANDLE_BRIDGE(_hdl, _func, _param, _hPCI, _card, _useBridge); \
	APPEND_DEBUG_LOG(_hdl, "> " #_func "() PCI_GET_HANDLE_LOCK hdl:%d", _hdl); \
	if (!(_useBridge) && !pclPciLock(_hdl)) { \
		APPEND_DEBUG_LOG(_hdl, "< " #_func "() PCI_GET_HANDLE_LOCK hdl:%d = PCL_ERR_TIMEOUT", _hdl); \
		return(PCL_ERR_TIMEOUT); \
	}

#define PCI_GET_HANDLE_LOCK(_hdl, _func, _param, _hPCI, _card)\
		PCI_GET_HANDLE_LOCK_BRIDGE(_hdl, _func, _param, _hPCI, _card, FALSE)

#define UNLOCK_RETURN_BRIDGE(_hdl, _func, _return, _useBridge) do { \
	if (!(_useBridge)) pclPciUnlock(_hdl); \
	APPEND_DEBUG_LOG(_hdl, "< " #_func "() UNLOCK_RETURN hdl:%d = " #_return, _hdl); \
	return(_return); \
} while (0)

#define UNLOCK_RETURN(_hdl, _func, _return)\
		UNLOCK_RETURN_BRIDGE(_hdl, _func, _return, FALSE)

//---------------------------------------------------------------------------

const char* _PCL_FDEF pclPciGetErrorString(		// @68, get error string
	void										// no parameters needed
);

//---------------------------------------------------------------------------

PCL_BOOL _PCL_FDEF pclPciExec(
	int hdl,
	BYTE cmd
);

enum {
	CMD_RESET							=  0x01,
	CMD_INT_MASK						=  0x03,
	CMD_SET_INT_MASK,					// 0x04,
	CMD_RESET_INT_MASK,					// 0x05,
	CMD_READER_ENABLE					=  0x10,
	CMD_VITC_SETUP_1					=  0x14,
	CMD_VITC_SETUP_2,					// 0x15,
	CMD_VITC_LEVEL,						// 0x16,
	CMD_LTC_ERROR,						// 0x17,
	CMD_VITC_ERROR,						// 0x18,
	CMD_FLY_WHEEL,						// 0x19,
	CMD_SET_REALTIME					=  0x1F,
	CMD_MIXED_ENABLE,					// 0x20,
	CMD_PRIORITY,						// 0x21,
	CMD_ATC_LTC_ERROR,					// 0x22,
	CMD_ATC_VITC_ERROR,					// 0x23,
	CMD_ATC_VITC2_ERROR,				// 0x24,	// obsolete
	CMD_HANC_LTC_ERROR,					// 0x25,
	CMD_HANC_VITC_ERROR,				// 0x26,
	CMD_GPI_OUT_1						=  0x28,
	CMD_GPI_OUT_2						=  0x2C,
	CMD_TC_FRAMES						=  0x30,
	CMD_LTC_FRAMES,						// 0x31,
	CMD_VITC_FRAMES,					// 0x32,
	CMD_GET_IDATA						=  0x38,
	CMD_SET_IDATA,						// 0x39,
	CMD_GET_PDATA,						// 0x3A,
	CMD_GET_XDATA						=  0x3C,
	CMD_GET_TC							=  0x41,
	CMD_GET_USER,						// 0x42,
	CMD_GET_REGISTER,					// 0x43,
	CMD_GET_REG_STATUS,					// 0x44,
	CMD_GET_VERSION						=  0x46,
	CMD_GET_VER_SPECIAL,				// 0x47,
	CMD_INT_ACK,						// 0x48,
	CMD_SET_DID,						// 0x49,
	CMD_SET_SDID,						// 0x4A,
	CMD_GET_VIDEO_STANDARD,				// 0x4B,
	CMD_GET_VIDEO_RASTER,				// 0x4C,
	CMD_SET_ANC_DID,					// 0x4D,
	CMD_GET_ANC_DATA,					// 0x4E,
	CMD_GET_GPI,						// 0x4F,
	CMD_GET_PAYLOAD_IDENTIFIER,			// 0x50,
	CMD_SET_USE_PAYLOAD_IDENTIFIER,		// 0x51,
	CMD_GET_ANC_DATA_TAB,				// 0x52,
	CMD_SET_GPI_EVENT,					// 0x53,
	CMD_GET_GPI_EVENT,					// 0x54,
	CMD_SET_GPI_IRQ,					// 0x55,
};

//---------------------------------------------------------------------------

int _PCL_FDEF pclIntEnable(	// enable or disable interrupts
	int hdl,				// in:	handle from pclPciOpen()
	PCL_BOOL enable			// in:	enable or disable interrupt
);

//---------------------------------------------------------------------------

int doResetPor(				// perform power-on reset
	int hdl					// in:	handle from pclPciOpen()
);

//---------------------------------------------------------------------------

#ifdef PCL_MULTI_MODE

TPciMultiCardTc *getMultiCardTc(int hdl, int imask);

int _PCL_FDEF pclMultiIgnored(void);

int _PCL_FDEF pclMultiNotSupported(void);

int imaskToReg(
	int imask					// in:	interrupt mask, like PCL_IMASK_xxx
);

int _PCL_FDEF pclPciReopenMulti(
	int hdl							// in:	handle from pclOpen()
);

#endif // PCL_MULTI_MODE

//---------------------------------------------------------------------------

#define DO_UNSUPPORTED(name)								{(void)hdl;return(PCL_ERR_NOT_SUPP);}
#define DO_UNSUPPORTED_1(name, p1)							{(void)p1;DO_UNSUPPORTED(name)}
#define DO_UNSUPPORTED_2(name, p1, p2)						{(void)p2;DO_UNSUPPORTED_1(name,p1)}
#define DO_UNSUPPORTED_3(name, p1, p2, p3)					{(void)p3;DO_UNSUPPORTED_2(name,p1,p2)}
#define DO_UNSUPPORTED_4(name, p1, p2, p3, p4)				{(void)p4;DO_UNSUPPORTED_3(name,p1,p2,p3)}
#define DO_UNSUPPORTED_5(name, p1, p2, p3, p4, p5)			{(void)p5;DO_UNSUPPORTED_4(name,p1,p2,p3,p4)}
#define DO_UNSUPPORTED_6(name, p1, p2, p3, p4, p5, p6)		{(void)p6;DO_UNSUPPORTED_5(name,p1,p2,p3,p4,p5)}
#define DO_UNSUPPORTED_7(name, p1, p2, p3, p4, p5, p6, p7)	{(void)p7;DO_UNSUPPORTED_6(name,p1,p2,p3,p4,p5,p6)}

//---------------------------------------------------------------------------

#ifdef PCL_DEBUG_LOG
	void appendDebugLog(int hdl, const char *format, ...);
#	ifdef LINUX
#		define APPEND_DEBUG_LOG(_hdl, _s, ...)	appendDebugLog(_hdl, "%s %4d: " _s, __FILE__, __LINE__, ##__VA_ARGS__)
#	else
#		define APPEND_DEBUG_LOG(_hdl, _s, ...)	appendDebugLog(_hdl, _s, __VA_ARGS__)
#	endif
	void GetTimeStamp(PCL_TIME_STAMP *stamp);
	void GetTimeStampFreq(PCL_TIME_STAMP *freq);
	void DiffTimeStamp(PCL_TIME_STAMP *diff, const PCL_TIME_STAMP *s1, const PCL_TIME_STAMP *s2);
	BOOL TimeStampIsZero(const PCL_TIME_STAMP *stamp);
	int TimeStampToMicro(const PCL_TIME_STAMP *stamp, const PCL_TIME_STAMP *freq);
#else
#	if defined _MSC_VER && _MSC_VER >= 1400 // MSVC 2005/2008
#		define APPEND_DEBUG_LOG(_hdl, _s, ...)	do; while (0)
#	elif defined LINUX
#		define APPEND_DEBUG_LOG(_hdl, _s, ...)	do {} while (0)
#	else
		inline void APPEND_DEBUG_LOG(int _hdl, const char *_s, ...)	{ }
#	endif
#endif // PCL_DEBUG_LOG

//---------------------------------------------------------------------------

#endif // _FUNCS_H
