/***************************************************************************\
*																			*
*	Windows DLL AVPCL32.DLL													*
*																			*
*	Copyright (c) 1995-2023 by PLURA Europe GmbH							*
*																			*
\***************************************************************************/

// $Id: global.h 1993 2023-01-05 03:00:53Z administrator $

#ifndef _PCL_GLOBAL_H
#define _PCL_GLOBAL_H

//---------------------------------------------------------------------------

#include "avpcl.h"
#ifdef _WIN32
#	if _MSC_VER < 1400 // older than MSVC 2005
#		undef PCL_OPT_VBI	/* read vbi lines */
#		undef PCL_OPT_ANC	/* read user defined ancillary data */
#	endif
#endif

//---------------------------------------------------------------------------

#ifndef LINUX
#	undef  _PCL_FDEF
#	define _PCL_FDEF         _stdcall
#	define _PCL_NDEF static _fastcall
#endif

//---------------------------------------------------------------------------

_PCL_EXTERN const char *pclErrorString;

//---------------------------------------------------------------------------

#define CHECK_PTR(ptr) \
{ \
	if ((ptr) == NULL) { \
		pclErrorString = "Parameter " #ptr " is NULL\n"; \
		return(PCL_ERR_NULL); \
	} \
} // check if handle '_hPCI' is plausible

//---------------------------------------------------------------------------

#define CHECK_RANGE(val, min, max, err) \
{ \
	if ((val) < (min)) { \
		sprintf_s(PCI_ErrorString, sizeof(PCI_ErrorString), "Parameter %s is less than %d\n", #val, min); \
		pclErrorString = PCI_ErrorString; \
		return err; \
	} \
	if ((val) > (max)) { \
		sprintf_s(PCI_ErrorString, sizeof(PCI_ErrorString), "Parameter %s is greater than %d\n", #val, max); \
		pclErrorString = PCI_ErrorString; \
		return err; \
	} \
} // check bounds of 'val' against 'min/max'

//---------------------------------------------------------------------------

void printfDebugString(const char *format, ...);

#define TRACE printfDebugString

//---------------------------------------------------------------------------

#if _MSC_VER <= 1100
int sprintf_s( char *buffer, size_t sizeOfBuffer, const char *format, ... );
int strncpy_s( char *strDest, size_t sizeOfBuffer, const char *strSource, size_t count );
#endif

//---------------------------------------------------------------------------

#if defined (LINUX)
#	define _MM_PAUSE do {__asm volatile ("pause" ::: "memory");} while (0)
#else
#	if _MSC_VER >= 1400  // MSVC 2005/8
#		define _MM_PAUSE _mm_pause()
//#		define _MM_PAUSE do {__asm{pause}} while (0)
#	else
#		define _MM_PAUSE do {__asm{_emit 0xf3}; __asm{_emit 0x90}} while (0)
#	endif // _MSC_VER >= 1400
#endif // LINUX

//---------------------------------------------------------------------------

#endif // _PCL_GLOBAL_H

//---------------------------------------------------------------------------
