/**********************************************\
*                                              *
*  Linux Kernel Driver for PCL PCI(e) Devices  *
*                                              *
*        Part of the shared library            *
*            (WinDriver-Clone)                 *
*                                              *
\**********************************************/

// $Id: pci_lib_linux.h 2123 2023-10-05 14:32:37Z J.Loh $
// Version: 5.45 (c) 1995-2023 PLURA Europe GmbH

#ifndef _PCI_LIB_LINUX_H_
#define _PCI_LIB_LINUX_H_

#include <asm/types.h>
#include "avpcl.h"
#include "driver.h"

//---------------------------------------------------------------------

typedef int PCI_HANDLE;
//typedef struct PCI_STRUCT *PCI_HANDLE;

#define PCI_ADDR unsigned long
#define WD_PCI_SLOT int


/*#ifndef INT
#define INT int
#endif
#ifndef PVOID
#define PVOID void *
#endif*/
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
// To use 64bit / 32bit operations, the
// makro do_div() from <asm/div64.h> must be used.
// The file funcs.cpp makes use of those functions.
//#include <asm/div64.h>
#ifndef __int64
#define __int64 long long
#endif
/*#ifndef NULL
#define NULL (void *)0
#endif*/

//---------------------------------------------------------------------

int sprintf_s(
   char *buffer,
   size_t sizeOfBuffer,
   const char *format, ... 
);

//---------------------------------------------------------------------

#define INTCSR_ACK_9030 0x0543				// Int ack
#define INTCSR_LINTI1_ENABLE_9030_M 0x0001	// LINTi1 Enable

//---------------------------------------------------------------------

#define	INTCSR_LINTI1_ENABLE_9056	0x0F010100	// interrupt enable
#define INTCSR_LINTI1_ENABLE_9056_M	0x00000800

//---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

//---------------------------------------------------------------------

typedef enum
{
	PCI_MODE_BYTE   = 0,
	PCI_MODE_WORD   = 1,
	PCI_MODE_DWORD  = 2
} PCI_MODE;

//---------------------------------------------------------------------

typedef struct
{
	DWORD dwCounter;	// number of interrupts received
	DWORD dwLost;		// number of interrupts yet to be handled
	BYTE  fStopped;		// was the interrupt disabled during wait
} PCI_INT_RESULT;

//---------------------------------------------------------------------

typedef void (*PCI_INT_HANDLER)(PCI_HANDLE hPCI, PCI_INT_RESULT *intResult);

//---------------------------------------------------------------------

/* PCI base address spaces (BARs) */
enum {
	AD_PCI_BAR0 = 0,
	AD_PCI_BAR1 = 1,
	AD_PCI_BAR2 = 2,
	AD_PCI_BAR3 = 3,
	AD_PCI_BAR4 = 4,
	AD_PCI_BAR5 = 5,
	AD_PCI_BARS = 6,
};

enum
{
	PCI_AD_BAR0 = AD_PCI_BAR0,
	PCI_AD_BAR1 = AD_PCI_BAR1,
	PCI_AD_BAR2 = AD_PCI_BAR2,
	PCI_AD_BAR3 = AD_PCI_BAR3,
	PCI_AD_BAR4 = AD_PCI_BAR4,
	PCI_AD_BAR5 = AD_PCI_BAR5,
//	PCI_AD_EPROM = AD_PCI_BAR_EPROM,
};

//---------------------------------------------------------------------

// PCI register definitions 
enum { PCI_DATA0_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA0_OFFSET = 0x0 };
enum { PCI_DATA1_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA1_OFFSET = 0x4 };
enum { PCI_DATA2_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA2_OFFSET = 0x8 };
enum { PCI_DATA3_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA3_OFFSET = 0xc };
enum { PCI_DATA4_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA4_OFFSET = 0x10 };
enum { PCI_DATA5_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA5_OFFSET = 0x14 };
enum { PCI_DATA6_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA6_OFFSET = 0x18 };
enum { PCI_DATA7_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA7_OFFSET = 0x1c };
enum { PCI_DATA8_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA8_OFFSET = 0x20 };
enum { PCI_DATA9_SPACE = PCI_AD_BAR2 };
enum { PCI_DATA9_OFFSET = 0x24 };
enum { PCI_DATAA_SPACE = PCI_AD_BAR2 };
enum { PCI_DATAA_OFFSET = 0x28 };
enum { PCI_INTFLGL_SPACE = PCI_AD_BAR2 };
enum { PCI_INTFLGL_OFFSET = 0x2c };
enum { PCI_CMDR_SPACE = PCI_AD_BAR2 };
enum { PCI_CMDR_OFFSET = 0x30 };
enum { PCI_ACK_SPACE = PCI_AD_BAR2 };
enum { PCI_ACK_OFFSET = 0x34 };
enum { PCI_CMDW_SPACE = PCI_AD_BAR2 };
enum { PCI_CMDW_OFFSET = 0x38 };
enum { PCI_CMDD0_SPACE = PCI_AD_BAR2 };
enum { PCI_CMDD0_OFFSET = 0x3c };
enum { PCI_CMDD1_SPACE = PCI_AD_BAR2 };
enum { PCI_CMDD1_OFFSET = 0x40 };
enum { PCI_CFG_SPACE = PCI_AD_BAR2 };
enum { PCI_CFG_OFFSET = 0x44 };
enum { PCI_INTFLGH_SPACE = PCI_AD_BAR2 };
enum { PCI_INTFLGH_OFFSET = 0x48 };
enum { PCI_VERSION_SPACE = PCI_AD_BAR2 };
enum { PCI_VERSION_OFFSET = 0x7c };
enum { PCI_INTCSR_9030_SPACE = PCI_AD_BAR0 };
enum { PCI_INTCSR_9030_OFFSET = 0x4c };
enum { PCI_CNTRL_SPACE = PCI_AD_BAR0 };
enum { PCI_CNTRL_OFFSET = 0x50 };
enum { PCI_EROMBRD_9030_SPACE = PCI_AD_BAR0 };
enum { PCI_EROMBRD_9030_OFFSET = 0x38 };
enum { PCI_INTCSR_9056_SPACE = PCI_AD_BAR0 };
enum { PCI_INTCSR_9056_OFFSET = 0x68 };

//---------------------------------------------------------------------

void Sleep(unsigned long timeout);

//---------------------------------------------------------------------

// 0x00  r  Data 0 (LSB)
BYTE PCI_ReadDATA0 (PCI_HANDLE hPCI);
// 0x01  r  Data 1
BYTE PCI_ReadDATA1 (PCI_HANDLE hPCI);
// 0x02  r  Data 2
BYTE PCI_ReadDATA2 (PCI_HANDLE hPCI);
// 0x03  r  Data 3
BYTE PCI_ReadDATA3 (PCI_HANDLE hPCI);
// 0x04  r  Data 4
BYTE PCI_ReadDATA4 (PCI_HANDLE hPCI);
// 0x05  r  Data 5
BYTE PCI_ReadDATA5 (PCI_HANDLE hPCI);
// 0x06  r  Data 6
BYTE PCI_ReadDATA6 (PCI_HANDLE hPCI);
// 0x07  r  Data 7
BYTE PCI_ReadDATA7 (PCI_HANDLE hPCI);
// 0x08  r  Data 8
BYTE PCI_ReadDATA8 (PCI_HANDLE hPCI);
// 0x09  r  Data 9
BYTE PCI_ReadDATA9 (PCI_HANDLE hPCI);
// 0x0A  r  Data A (MSB)
BYTE PCI_ReadDATAA (PCI_HANDLE hPCI);
// 0x0C  r  Command Response
BYTE PCI_ReadCMDR (PCI_HANDLE hPCI);
// 0x0D  r  Acknowledge
BYTE PCI_ReadACK (PCI_HANDLE hPCI);
// 0x0E w Command
void PCI_WriteCMDW (PCI_HANDLE hPCI, BYTE data);
// 0x0F w Command Data 0
void PCI_WriteCMDD0 (PCI_HANDLE hPCI, BYTE data);
// 0x10 w Command Data 1
void PCI_WriteCMDD1 (PCI_HANDLE hPCI, BYTE data);
// 0x11 w Config
void PCI_WriteCFG (PCI_HANDLE hPCI, BYTE data);
// 0x1F rw Version Number
BYTE PCI_ReadVERSION (PCI_HANDLE hPCI);
// 0x0B  r  Interrupt Flags
BYTE PCI_ReadINTFLGL (PCI_HANDLE hPCI);
BYTE PCI_ReadINTFLGH (PCI_HANDLE hPCI);
// 0x13  w  Interrupt Acknowledge
void PCI_WriteINT_ACK (PCI_HANDLE hPCI);

// PLX rw Interrupt Control/Status Register
// DWORD PCI_ReadINTCSR (PCI_HANDLE hPCI);
void PCI_WriteINTCSR (PCI_HANDLE hPCI, DWORD data);

int PCI_GetDeviceId(PCI_HANDLE hPCI, WORD _PCL_OUT *result);
int PCI_GetSubsystemDeviceId(PCI_HANDLE hPCI, WORD _PCL_OUT *result);

int PCI_GetDeviceIdBridge(PCI_HANDLE hPCI, WORD _PCL_OUT *result);

#ifdef PCL_OPT_FLASH
int PCI_GetDeviceInfo(PCI_HANDLE hPCI, TDeviceInfo _PCL_OUT *deviceInfo);
int PCI_GetDeviceInfoBridge(PCI_HANDLE hPCI, TDeviceInfo _PCL_OUT *deviceInfo);
int PCI_GetBus(PCI_HANDLE hPCI, BYTE _PCL_OUT *bus);
int PCI_ReadOffset8 		(PCI_HANDLE hPCI, BYTE bar, UINT offset, BYTE _PCL_OUT *result);
int PCI_ReadOffset8Bridge	(PCI_HANDLE hPCI, BYTE bar, UINT offset, BYTE _PCL_OUT *result);
int PCI_ReadOffset32		(PCI_HANDLE hPCI, BYTE bar, UINT offset, DWORD _PCL_OUT *result);
int PCI_ReadOffset32Bridge	(PCI_HANDLE hPCI, BYTE bar, UINT offset, DWORD _PCL_OUT *result);
int PCI_WriteOffset8 		(PCI_HANDLE hPCI, BYTE bar, UINT offset, BYTE  data);
int PCI_WriteOffset8Bridge	(PCI_HANDLE hPCI, BYTE bar, UINT offset, BYTE  data);
int PCI_WriteOffset32		(PCI_HANDLE hPCI, BYTE bar, UINT offset, DWORD data);
int PCI_WriteOffset32Bridge	(PCI_HANDLE hPCI, BYTE bar, UINT offset, DWORD data);
int PCI_ConfigDump(PCI_HANDLE hPCI, UINT offset, UINT size, void _PCL_OUT *result);
int PCI_ConfigDumpBridge(PCI_HANDLE hPCI, UINT offset, UINT size, void _PCL_OUT *result);
BOOL PCI_IsBridge(PCI_HANDLE hPCI);
#endif

// Interrupt control
void PCI_WriteEnable(int hdl);
void PCI_WriteDisable(int hdl);
BOOL PCI_IntIsEnabled (PCI_HANDLE hPCI);
void PCI_IntDisable (PCI_HANDLE hPCI);
BOOL IntEnable (int hdl, PCI_INT_HANDLER funcIntHandler);

// get card filename
const char *PCL_CardFilename(int card);

// this string is set to an error message, if one occurs
extern char PCI_ErrorString[1024];
//extern char pclErrorString[1024];

#ifdef __cplusplus
}
#endif

//---------------------------------------------------------------------

void PCI_ERR(const char *format, ...);

#endif
