﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using AvPcl;

namespace AvPclNETTest
{
    public partial class Form1 : Form
    {
        //#####################################################################
        #region Member
        private bool running = false;
        AvPcl.CardHandler m_Card = new AvPcl.CardHandler();
        #endregion

        //#####################################################################
        #region Initialize, Deinitialize
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            List<AvPcl.CardHandler.BaseAddress> cardList = FindCards();
            foreach (AvPcl.CardHandler.BaseAddress ba in cardList)
            {
                listBoxCards.Items.Add(ba);
            }
            button1.Enabled = false;

            if (listBoxCards.Items.Count > 0)
                listBoxCards.SelectedIndex = 0;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
        }
        #endregion


        //#####################################################################
        #region GUI Update
        private delegate void MethodWParam(TextBox tb, String str);

        private void SetTextboxText(TextBox tb, String txt)
        {
            if (tb.InvokeRequired)
            {
                tb.Invoke(new MethodWParam(SetTextboxText), new object[] { tb, txt });
            }
            else
            {
                tb.Text = txt;
            }
        }
        #endregion

        //#####################################################################
        #region Card Access

        private List<AvPcl.CardHandler.BaseAddress> FindCards()
        {
            List<AvPcl.CardHandler.BaseAddress> allCards = new List<AvPcl.CardHandler.BaseAddress>();

            AvPcl.CardHandler cardHandler = new AvPcl.CardHandler();

            uint baseAddress = (uint)AvPcl.CardHandler.BaseAddress.Pci01;
            int maxHandleCount = AvPcl.CardHandler.BaseAddress.PciLast - AvPcl.CardHandler.BaseAddress.PciOffset;

            for (int n = 0; n < maxHandleCount; n++)
            {
                AvPcl.CardHandler.BaseAddress ba = (AvPcl.CardHandler.BaseAddress) baseAddress++;
                AvPcl.CardHandler.ReturnCode ret = cardHandler.Open(ba);
                if (ret == AvPcl.CardHandler.ReturnCode.Ok)
                {
                    allCards.Add(ba);
                    cardHandler.Close();
                }
                else
                {
                    if (ret != AvPcl.CardHandler.ReturnCode.ErrorNotFound)
                        MessageBox.Show("Error: " + ret);
                    break;
                }
            }

            return allCards;
        }

        private void listBoxCards_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listBoxCards.SelectedItem != null)
                button1.Enabled = true;
        }

        private void ShowTimecode(AvPcl.Timecode tc)
        {
            SetTextboxText(textBox1, tc.ToString());
        }

        private void Isr(AvPcl.CardHandler card, int flag, AvPcl.Dll.PclIntResult intResult)
        {
            AvPcl.Timecode tc;
            switch ((CardHandler.InterruptMask)flag)
            {
                case CardHandler.InterruptMask.Mixed:
                    tc = card.GetTc();
                    break;
                case CardHandler.InterruptMask.Ltc:
                    tc = card.GetRegister(CardHandler.Register.Ltc);
                    break;
                case CardHandler.InterruptMask.Vitc:
                    tc = card.GetRegister(CardHandler.Register.Vitc);
                    break;
                case CardHandler.InterruptMask.AtcLtc:
                    tc = card.GetRegister(CardHandler.Register.AtcLtc);
                    break;
                case CardHandler.InterruptMask.AtcVitc:
                    tc = card.GetRegister(CardHandler.Register.AtcVitc);
                    break;
                default:
                    tc = new Timecode();
                    break;
            }
            ShowTimecode(tc);
            SetTextboxText(textBox2, (CardHandler.InterruptMask)flag + "");
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            // Read timecode from Mixed register in this example.
            AvPcl.Timecode tc = m_Card.GetTc();

            // To read from other timecode register change it to the following:
            //      AvPcl.Timecode tc = m_Card.GetRegister(CardHandler.Register.AtcLtc);
            // Possible values are:
            // Mixed, Ltc, Vitc, AtcLtc, AtcVitc
            // (see definition of Register)

            ShowTimecode(tc);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // This sample is prepared for either interrupt or polling mode
            // (see comments below)
            bool bUseInterrupt = true;

            if (!running)
            {
                running = true;
                listBoxCards.Enabled = false;
                button1.Text = "&Close";
                AvPcl.CardHandler.ReturnCode ret;
                do
                {
                    ret = m_Card.Open((AvPcl.CardHandler.BaseAddress)listBoxCards.SelectedItem);
                    if (ret != AvPcl.CardHandler.ReturnCode.Ok) break;

                    if (bUseInterrupt)
                    {
                        // ------------------------------------------------------
                        // Initialize interrupt mode
                        ret = m_Card.SetIntHandler(Isr);
                        if (ret != AvPcl.CardHandler.ReturnCode.Ok) break;

                        // Enable LTC interrupts in this example.
                        // Possible interrupts are:
                        // Mixed, Ltc, Vitc, AtcLtc, AtcVitc
                        // (see definition of InterruptMask)
                        ret = m_Card.SetIntMask(AvPcl.CardHandler.InterruptMask.Ltc);
                        if (ret != AvPcl.CardHandler.ReturnCode.Ok) break;
                        // ------------------------------------------------------
                    }
                    else
                    {
                        // ------------------------------------------------------
                        // Initialize polling mode
                        timer1.Interval = 40;   // poll every 40 milliseconds
                        timer1.Start();
                        // ------------------------------------------------------
                    }

                } while (false);

                if (ret != AvPcl.CardHandler.ReturnCode.Ok)
                {
                    MessageBox.Show("Failed: " + ret, "Error");
                    ret = m_Card.Close();
                    if (ret != AvPcl.CardHandler.ReturnCode.Ok)
                        MessageBox.Show("Couldn't close card: " + ret, "Error");
                }
            }
            else
            {
                running = false;
                listBoxCards.Enabled = true;
                button1.Text = "&Open";

                timer1.Stop();

                m_Card.Close();
            }
        }

        #endregion
    }
}
